/* discover.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <glade/glade-xml.h>
#include <gtk/gtk.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <pwd.h>
#include "main.h"
#include "misc.h"
#include "task.h"
#include "session.h"
#include "preferences.h"
#include "scheduler.h"
#include "gui.h"
#include "discover.h"
#include "filtermodule.h"
#include "licon.xpm"
#include "ssummary.xpm"

/***************************************************************************
 Callbacks
 ***************************************************************************/

static void close_cb(GtkWidget *w, gpointer data)
{ ((Discover*)data)->hide(); }
static void start_cb(GtkWidget *w, gpointer data)
{ ((Discover*)data)->start(); ((Discover*)data)->hide(); }
static void community_cb(GtkWidget *w, gpointer data)
{ ((Discover*)data)->main->gui->preferences->show_community(); }
static void filterstate_cb(GtkWidget *w, gpointer data)
{ gtk_widget_set_sensitive(((Discover*)data)->vbox11, ((GTK_TOGGLE_BUTTON(w)->active)?TRUE:FALSE)); }
static void restrict_cb(GtkWidget *w, gpointer data)
{ gtk_widget_set_sensitive(((Discover*)data)->spinbutton1, ((GTK_TOGGLE_BUTTON(w)->active)?TRUE:FALSE)); }
static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{ ((Discover*)data)->hide(); return TRUE; }

/***************************************************************************
 Discover Class
 ***************************************************************************/

Discover::Discover(Main *m)
{
	GtkObject *spinbutton_adj;
	GladeXML *gxml;

	main = m;

	gchar *fname = user_file("nomad.glade");
	gxml = glade_xml_new (fname, "discover");
	if (!gxml)
	{
		g_warning("Failed to load file nomad.glade");
		return;
	}

	window = glade_xml_get_widget (gxml, "discover");
	frame9 = glade_xml_get_widget (gxml, "frame9");
	button14 = glade_xml_get_widget (gxml, "button14");
	button15 = glade_xml_get_widget (gxml, "button15");
	button16 = glade_xml_get_widget (gxml, "button16");
	radiobutton2 = glade_xml_get_widget (gxml, "radiobutton2");
	checkbutton4 = glade_xml_get_widget (gxml, "checkbutton4");
	checkbutton6 = glade_xml_get_widget (gxml, "checkbutton6");
	checkbutton7 = glade_xml_get_widget (gxml, "checkbutton7");
	spinbutton1 = glade_xml_get_widget (gxml, "spinbutton1");
	vbox11 = glade_xml_get_widget (gxml, "vbox11");
	gtk_object_unref (GTK_OBJECT (gxml));
	g_free(fname);

	gtk_widget_realize (window);
	window_icon(window, licon_xpm);
	gdk_window_set_icon_name(window->window, "Task List");
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
		GTK_SIGNAL_FUNC(delete_event), this);

	pix[0] = create_pixmap(window, ssummary_xpm);
	gtk_container_add (GTK_CONTAINER (button16), pix[0]);

	filtermodule = new FilterModule(window);
	gtk_container_add (GTK_CONTAINER (frame9), filtermodule->vbox58);
  gtk_container_set_border_width (GTK_CONTAINER (filtermodule->vbox58), 4);
//	gtk_box_pack_start (GTK_BOX (vbox11), filtermodule->vbox58, TRUE, TRUE, 0);
//	gtk_box_reorder_child (GTK_BOX (vbox11), filtermodule->vbox58, 0);

  spinbutton_adj = gtk_adjustment_new(3, 1, 32, 1, 1, 1);
	gtk_spin_button_set_adjustment(GTK_SPIN_BUTTON(spinbutton1),
		GTK_ADJUSTMENT(spinbutton_adj));

	gtk_signal_connect (GTK_OBJECT (checkbutton4), "clicked",
		GTK_SIGNAL_FUNC (filterstate_cb), this);
	gtk_signal_connect (GTK_OBJECT (checkbutton6), "clicked",
		GTK_SIGNAL_FUNC (restrict_cb), this);
	gtk_signal_connect (GTK_OBJECT (button16), "clicked",
		GTK_SIGNAL_FUNC (community_cb), this);
	gtk_signal_connect (GTK_OBJECT (button14), "clicked",
		GTK_SIGNAL_FUNC (start_cb), this);
	gtk_signal_connect (GTK_OBJECT (button15), "clicked",
		GTK_SIGNAL_FUNC (close_cb), this);

	get_gateway();
}

Discover::~Discover()
{
	if (filtermodule!=NULL) delete filtermodule;
}

void Discover::show()
{
	char buf[256];

	gtk_widget_show(window);
	gdk_window_raise(window->window);

	sprintf(buf, "Default Gateway (%s)", gateway.getstring());
	gtk_label_set_text(GTK_LABEL(GTK_BIN (radiobutton2)->child), buf);
//	update_discover();
}

void Discover::hide()
{
	gtk_widget_hide(window);
}

void Discover::start()
{
	char buf[1024];
	sprintf(buf, "%s %s", gateway.getstring(),
		get_community_list_string(main->client));
	main->scheduler->addTask(buf, 1, 3, 4);
//	main->scheduler->addTask(buf, 1, 3, insert_hop_here);
//	sprintf(buf, "xxx.xxx.xxx.xxx %s",
//	main->session->options->getCommunityList());
//	main->scheduler->addTask(buf, 1, 2, 3);
}

void Discover::get_gateway()
{
	FILE *f;
	char buff[1024], junk[1024];
	char addr1[16], addr2[16], addr3[16];
	int val;

	f = fopen("/proc/net/route", "r");

	gateway.empty();
	if (f)
	{
		fgets(buff, 1023, f);
		while (fgets(buff, 1023, f))
		{
			sscanf(buff, "%[^\t]\t%[^\t]\t%[^\t]\t%X\t%[^\n]\n",
				addr1, addr2, addr3, &val, junk);
			if ((val & 2)==2)
			{
				gateway.setafinet(addr3);
			}
		}
		fclose(f);
	}
}

int Discover::match(IPAddress ip, int hop)
{
	gboolean value;

	value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton6));
	if ((value) && (hop==0)) return 0;

	value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton7));
	// scan through list of devices/interfaces/ip addresses
	// if ((!value) && (match)) return 0;

	value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton4));
	if ((value) && (!filtermodule->match(ip))) return 0;

	return 1;
}

