/* main.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <glade/glade.h>
#include <gnome.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <pwd.h>
#include "header.h"
#include "main.h"
#include "gui.h"
#include "preferences.h"
#include "monitor.h"
#include "session.h"
#include "scheduler.h"
#include "topology.h"

int main(int argc, char *argv[])
{
	Main m(argc, argv);
	return 0;
}

gint timeout_cb(gpointer data)
{ ((Main*)data)->timeout_callback(); return TRUE; }
gint init_cb(gpointer data)
{ ((Main*)data)->init_callback(); return FALSE; }
gint refresh_cb(gpointer data)
{ ((Main*)data)->refresh_callback(); return TRUE; }

Main::Main(int argc, char *argv[])
{
	GError *err;
	gchar *teststr;

	scheduler = new Scheduler(this);
	topology = new Topology(this);

	gnome_init(PACKAGE, VERSION, argc, argv);
	gconf_init (argc, argv, NULL);
	glade_gnome_init();
	gdk_rgb_init ();
	client = gconf_client_get_default();
  gconf_client_add_dir(client, "/apps/nomad",
		GCONF_CLIENT_PRELOAD_NONE, &err);
	session = new Session(this);
	gui = new Gui(this);

	/* check if gconf schemas are working */
	teststr = gconf_client_get_string (client, CONFIG_TEST, &err);
	if (!teststr)
	{
		GtkWidget *dialog;
		dialog = gnome_error_dialog (N_("Can't find a schema for nomad preferences.\nCheck your gconf setup"));
		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
		exit (0);
	}
	else
	{
		g_free (teststr);
	}

	gtk_timeout_add(2000, timeout_cb, this);
	gtk_timeout_add(0, init_cb, this);
	gtk_timeout_add(30, refresh_cb, this);
	gtk_main();
}

Main::~Main()
{
	if (session!=NULL) delete session;
	if (scheduler!=NULL) delete scheduler;
	if (topology!=NULL) delete topology;
	if (gui!=NULL) delete gui;
}

void Main::init_callback()
{
	struct stat statbuf;
	GError *error;
	gchar *str;
	char buf[256];

	gui->preferences->global_init();
	gui->monitor->show();
	update();
	setTitle();
	// FIXME this variable should be loaded along side the global init
	str = gconf_client_get_string (client, CONFIG_SESSION_STARTUP, &error);
	if ((str!=NULL) && (strcmp(str, "")!=0))
	{
		sprintf(buf, "%s/sessions/%s", session->datadir, str);
		if (stat(buf, &statbuf) != -1)
			session->load_session(str);
	}
	g_free(str);
}

void Main::timeout_callback()
{
	scheduler->timeout();
}

void Main::refresh_callback()
{ gui->monitor->refresh(); }

void Main::setTitle()
{
	char buf[256], flag1[4], flag2[4];

	strcpy(flag1, ((session->devicebit==TRUE) || (session->positionbit==TRUE))?"* ":"");
	strcpy(flag2, (strcmp(session->session_name, "")!=0)?" - ":"");
	sprintf(buf, "%sNomad%s%s", flag1, flag2, session->session_name);
	gtk_window_set_title(GTK_WINDOW(gui->monitor->mainwindow), buf);
}

void Main::update()
{
	gui->update_lists();
}

void Main::quit()
{
	gtk_exit(0);
}

