/* graphio.h
 * Copyright (C) 2001-2 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef GRAPHIO_H
#define GRAPHIO_H 1

#include <glib.h>
#include "dataset.h"
#include "header.h"

class GraphBlock
{
	public:
		GraphBlock();

		unsigned long gap;				// 5 minute intervals
		unsigned long howlong;		// upto 2 weeks
		unsigned long length;			// block is x bytes long
		unsigned long start;			// start of block is x bytes into the file
		unsigned long offset;			// the next record should be written here
		unsigned long average;		// position of average record to smooth data
		unsigned long time;				// length of time for average
		gboolean wrap;
};

class GraphIO
{
	public:
		GraphIO();
		~GraphIO();

		void init_local();
		int valid();							// does the local match gconf values
		// load header from file and store in local
		void load_header(FILE *f);
		int create(char *fname);		// create new file with local values
		int add(char *fname);								// add value data to file
		gboolean get(char *fname, DataSet *d);		// load file to dataset suitable 
		void print(char *fname);

		char datadir[256];
		int num_values;
		int record_size;
		gboolean replace_flag;

		GraphBlock global_blocks[GRAPH_MAX_BLOCKS];
		int num_global_blocks;

		GraphBlock blocks[GRAPH_MAX_BLOCKS];
		int num_blocks;

		// fill these in before calling add
		TimeStamp current;
		char status;
		unsigned long value[GRAPH_NUM_TOTAL];

	private:
		void add_recursive(FILE *f, int lvl);
		void get_record(DataSet *d, FILE *f, int count);
};

#endif

